@@ -29,6 +29,10 @@
 #ifndef CLIENTTHREAD_H
 #define CLIENTTHREAD_H
 
+#include "Dbus/gtkbindings-new.h"
+#include <libosso.h>
+#include <libosso-abook/osso-abook.h>
+
 #include <QThread>
 #include <QString>
 #include <QTcpSocket>
@@ -48,6 +52,8 @@
 #include "Whatsapp/connection.h"
 #include "Whatsapp/phonereg.h"
 
+#include "Whatsapp/util/datacounters.h"
+
 #include "Contacts/contactroster.h"
 #include "Contacts/contactsyncer.h"
 
@@ -72,8 +78,19 @@ public:
         RegistrationFailed
     };
 
+    enum SyncFrequencies {
+        onConnect,
+        onceADay,
+        onceAWeek,
+        onceAMonth
+    };
+
+    static DataCounters dataCounters;
+
     static ConnectionStatus connectionStatus;
 
+    static osso_context_t *osso_context;
+
     // Settings
     static QSettings *settings;
 
@@ -94,6 +111,11 @@ public:
     static QString userName;
     static QString imei;
 
+    static QString creation;
+    static QString expiration;
+    static QString kind;
+    static QString accountstatus;
+
     // Port
     static quint16 port;
 
@@ -115,22 +137,29 @@ public:
     // Import media into gallery
     static bool importMediaToGallery;
 
+    // Last time photo refresh was performed
+    static qint64 lastSync;
+
     // What's new window
-    static qint64 whatsnew;
+    static qint64 whatsNew;
 
     // Sync
     static QString sync;
 
+    // Sync frequency
+    static int syncFreq;
+
+    // Start on boot
+    static bool startOnBoot;
+
     // Android password encryption method
     static bool android;
 
     static bool isSynchronizing;
 
-    explicit Client(QObject *parent = 0);
+    explicit Client(bool minimized, QObject *parent = 0);
     ~Client();
 
-    static QString getPathFor(int media_wa_type, bool gallery);
-
 private:
     MainWindow *mainWin;
     Connection *connection;
@@ -143,6 +172,7 @@ private:
     DBusNokiaMCESignalIf *mceAdaptor;
     DBusAppletIf *applet;
     ContactSyncer *syncer;
+    qint64 lastCountersWrite;
 
     // Timers
     QTimer *keepAliveTimer;
@@ -191,12 +221,20 @@ public slots:
     void sendSetGroupSubject(QString gjid, QString subject);
     void requestLeaveGroup(QString jid);
     void requestQueryLastOnline(QString jid);
-    void userStatusUpdated(QString status);
+    void userStatusUpdated(FMessage message);
     void changeStatus(QString newStatus);
     void changeUserName(QString newUserName);
     void synchronizeContacts();
     void syncFinished();
     void syncProgress(int progress);
+    void photoRefresh(QString jid, QString expectedPhotoId, bool largeFormat);
+    void photoDeleted(QString jid);
+    void photoIdReceived(QString jid, QString pictureId);
+    void photoReceived(QString from, QByteArray data,
+                       QString photoId, bool largeFormat);
+    void requestContactStatus(QString jid);
+    void setPhoto(QImage image);
+
 
 public Q_SLOTS:
     void ShowWindow();
